
// SimpleSoundLoc.java
// Andrew Davison, March 2012, ad@fivedots.coe.psu.ac.th

/* A simple non-GUI example of how to use Laurent Calmes' sound localizer,
   originally downloaded from http://www.laurentcalmes.lu
*/

import java.io.*;
import java.util.*;
import javax.sound.sampled.*;

import com.laurentcalmes.soundloc.*;    // from SL.jar



public class SimpleSoundLoc
{
  private static final String INPUT_MIXER = "kinect";
                               // "sigmatel";   //"realtek";  // "kinect";


  public static void main(String[] args) throws Exception
  {
    Mixer.Info mi = getTarget(INPUT_MIXER);

    double tfSize = 0.062;       // timeframe size (in secs)
    AudioFormat af = new AudioFormat(16000.0f, 16, 2, true, false);
              // sampleRate, sample size in bits, no. channels, is signed, is bigEndian

    HWDevice hwd = new HWDevice(tfSize, af, mi);
                    // Timeframe, AudioFormat, hardware device
    hwd.init();

    LocalizationAlgorithm alg = initAlg(tfSize, hwd);

    List<LocalizationAlgorithm.Az> peaks;    
               // list of detected sound sources in decreasing conf order
    while (true) {      // user must type ctrl-c to exit the app
      peaks = alg.compute();
      reportPeaks(peaks);
    }
  }  // end of main()



  private static Mixer.Info getTarget(String nm)
  /* look at all the mixers, and return the first one that contains nm
     in its name and has suitable target lines
  */
  { 
    Mixer mixer =  AudioUtils.getNamedTargetLine(nm);
    if (mixer == null) {
      System.out.println("No mixer using \"" + nm +"\" found");
      System.exit(1);
    }

    Mixer.Info mi = mixer.getMixerInfo();
    System.out.println("Found \"" + mi.getName() + "\"");
    return mi;
  }  // end of getTarget()




  private static LocalizationAlgorithm initAlg(double tfSize, HWDevice hwd)
                                                                throws Exception
  {
    // CorrelationAlgorithm mAlg =  new CorrelationAlgorithm(0.226, 0.01, tfSize, hwd);
            /* mike dist in meters, integration time in seconds
               timeframe size in seconds, Audio device
            */

    int tfSizeSpls = (int) Math.round(tfSize * hwd.getFormat().getSampleRate());
    int logSize = FFT.log2(tfSizeSpls);
    int fftSize = 1 << logSize;
    if (fftSize < tfSizeSpls)
      fftSize <<= 1;
    DDLAlgorithm mAlg = new DDLAlgorithm(0.226, 361, 0.88, fftSize, tfSize, hwd);
             /* mic distance in metres, number of delays in the delay line,
                coincidence map time integration time constant in secs,
                size of the FFT in samples, timeframe size in secs,
                audio device
             */
    mAlg.init();

    return mAlg;
  }  // end of initAlg()



  private static void reportPeaks(List<LocalizationAlgorithm.Az> peaks)
  {
    System.out.print("(angle, conf): ");
    for (LocalizationAlgorithm.Az az : peaks) {
      double rotAng = az.getAzimuth();
      double conf = az.getHeight();      // algorithm-dependent confidence value
      System.out.printf("(%.1f, %.2f) ", rotAng, conf);
    }
    System.out.println();
  }  // end of reportPeaks()


}  // end of SimpleSoundLoc class

