
// CaptureMixers.java
// Andrew Davison, March 2012, ad@fivedots.coe.psu.ac.th

/* Print out information on all the capture mixers.

   An capture mixer is either:
         1) a TargetDataLine  (for incoming audio):
      or
         2) for a port on a mixer source line (for the recording controls) 
*/


import java.util.*;
import javax.sound.sampled.*;


public class CaptureMixers
{
  public static void main(String[] args)
  {
    Mixer defaultMixer = AudioSystem.getMixer(null);
    if (defaultMixer == null) {
      System.out.println("Audio system unavailable");
      return;
    }

    System.out.println("Is there microphone support? " +
                             AudioSystem.isLineSupported(Port.Info.MICROPHONE));
    System.out.println("Is there a microphone source? " + Port.Info.MICROPHONE.isSource());
    System.out.println();

    System.out.println("Looking for Capture Mixers...\n");
    int numCaptures = reportCaptures();
    if (numCaptures == 0)
      System.out.println("None found");
  }  // end of main()



  private static int reportCaptures()
  /*  An capture mixer is either:
         1) a TargetDataLine  (for incoming audio):
      or
         2) for a port on a mixer source line (for the recording controls)
  */
  {
    int numCaptures = 0;
    for (Mixer.Info mi : AudioSystem.getMixerInfo()) {
      Mixer mixer = AudioSystem.getMixer(mi);
      if (AudioUtils.isTargetDataLine(mixer) || 
          AudioUtils.isInputPort(mixer)) {
        numCaptures++;
        printMixerInfo(numCaptures, mixer);
      }
    }
    return numCaptures;
  }  // end of reportCaptures()





  private static void printMixerInfo(int idNum, Mixer mixer)
  // print the mixer's name, description, and source and target line info
  {
    AudioUtils.printInfo(idNum, mixer);

    for (Line.Info lineInfo : mixer.getSourceLineInfo())
      System.out.println("   Source line: " + lineInfo);
    for (Line.Info lineInfo : mixer.getTargetLineInfo())
      System.out.println("   Target line: " + lineInfo);

    System.out.println();
  }  // end of printMixerInfo()


}  // end of CaptureMixers class