
// ListMixers.java
// Andrew Davison, March 2012, ad@fivedots.coe.psu.ac.th

/* Print out all the mixers, including their names, descriptions, 
   and types of audio lines they support.

   A Mixer is a logical container for SourceDataLines, TargetDataLines, or Ports. 
*/

import java.util.*;
import java.io.*;
import javax.sound.sampled.*;



public class ListMixers
{

  public static void main(String[] args)
  {
    Mixer defaultMixer = AudioSystem.getMixer(null);
    if (defaultMixer == null) {
      System.out.println("Audio system unavailable");
      return;
    }
    else
      System.out.println("Default mixer: " + defaultMixer.getMixerInfo());

    Mixer.Info[] mis = AudioSystem.getMixerInfo();
    System.out.println("No. mixers: " + mis.length);
    for (int i = 0; i < mis.length; i++) {
      Mixer mixer = AudioSystem.getMixer(mis[i]);
      System.out.println();
      AudioUtils.printInfo((i+1), mixer);
      printLines(mixer);
    }
  }  // end of main()



  private static void printLines(Mixer mixer)
  // print the line types supported by this mixer
  {
    String support = "";
    if (AudioUtils.isSourceDataLine(mixer))
      support += "SourceDataLine; ";

    if (AudioUtils.isClip(mixer))
      support += "Clip; ";

    if (AudioUtils.isTargetDataLine(mixer))
      support += "TargetDataLine; ";

    if (AudioUtils.isInputPort(mixer))
      support += "Input Port; ";

    if (AudioUtils.isOutputPort(mixer))
      support += "Output Port; ";


    System.out.println("   Lines: " + support);
  }  // end of printLines()

}  // end of ListMixers class

