/* SoundLocalizer - a sound source localization program
 * Copyright (C) 2010 Laurent Calmes
 *
 * This file is part of SoundLocalizer.
 *
 * SoundLocalizer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SoundLocalizer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SoundLocalizer.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.laurentcalmes.soundloc;

import java.io.IOException;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.TargetDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;

/**
 * Implements an <tt>AudioDevice</tt> reading from the system's audio hardware.
 * 
 * @author Laurent Calmes
 * 
 */
public class HWDevice extends AudioDevice {
	private TargetDataLine mLine;
	private AudioInputStream mIStream;

	/**
	 * 
	 * 
	 * @param tfSize
	 *            Timeframe size in seconds
	 * @param af
	 *            Desired <tt>AudioFormat</tt>
	 * @param mixer
	 *            Desired hardware device
	 * @throws IOException
	 * @throws UnsupportedAudioFileException
	 * @throws LineUnavailableException
	 */
	public HWDevice(double tfSize, AudioFormat af, Mixer.Info mixer)
			throws IOException, UnsupportedAudioFileException,
			LineUnavailableException {
		super(tfSize);
		mLine = AudioSystem.getTargetDataLine(af, mixer);
		mLine.open();
		mLine.start();
		mIStream = new AudioInputStream(mLine);
	}

	@Override
	public AudioFormat getFormat() {
		return mIStream.getFormat();
	}

	@Override
	protected void readRaw(byte[] rawData) throws IOException {
		int read = mIStream.read(rawData, 0, rawData.length);
		if (read < 0) { // EOF reached
			// in case of reading from audio device, sth. bad happened and
			// we should exit
			throw new IOException("HWDevice.readRaw(): Audio bytes requested: "
					+ rawData.length + "; read: " + read);
		}
	}

}
