/* SoundLocalizer - a sound source localization program
 * Copyright (C) 2010 Laurent Calmes
 *
 * This file is part of SoundLocalizer.
 *
 * SoundLocalizer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * SoundLocalizer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SoundLocalizer.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.laurentcalmes.soundloc;

import java.io.File;
import java.io.IOException;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.UnsupportedAudioFileException;

/**
 * Implements an <tt>AudioDevice</tt> reading from a specified WAV file.
 * 
 * @author Laurent Calmes
 * 
 */
public class WavDevice extends AudioDevice {
	private AudioInputStream mIStream;
	private File mWavFile;

	/**
	 * @param tfSize
	 *            Timeframe size in seconds
	 * @param wavFileName
	 *            File name of the WAV file from which to read
	 * @throws IOException
	 * @throws UnsupportedAudioFileException
	 */
	WavDevice(double tfSize, String wavFileName) throws IOException,
			UnsupportedAudioFileException {
		super(tfSize);
		mWavFile = new File(wavFileName);
		mIStream = AudioSystem.getAudioInputStream(mWavFile);
	}

	@Override
	public AudioFormat getFormat() {
		return mIStream.getFormat();
	}

	@Override
	protected void readRaw(byte[] rawData) throws IOException {
		assert rawData.length != 0 : "rawData.length == 0!";
		int read = mIStream.read(rawData, 0, rawData.length);
		if (read < 0) { // EOF reached
			// close and reopen to restart from beginning
			try {
				mIStream.close();
				mIStream = AudioSystem.getAudioInputStream(mWavFile);
			} catch (Exception e) {
				// as we already read from the file (so it exists and is
				// valid), if reading fails at this point something's really
				// wrong and we better exit
				e.printStackTrace();
				System.exit(1);
			}
		}
	}

}
